/* This file was generated by syncqt. */
#ifndef QT_QTFEEDBACK_VERSION_H
#define QT_QTFEEDBACK_VERSION_H

#define QTFEEDBACK_VERSION_STR "0.0.0"

#define QTFEEDBACK_VERSION 0x000000

#endif // QT_QTFEEDBACK_VERSION_H
